@@ -1,4 +1,30 @@
-"""Probability and Likelihood Functions for Distribution Testing"""
+"""Probability and Likelihood Functions for Distribution Testing
+
+Probability distributions
+    Poisson lognormal distribution
+    Upper truncated logseries distribution
+    Lower truncated exponential distribution
+    Lower truncated Pareto (power) distribution
+    Lower truncated Weibull distribution
+    Upper truncated geometric distribution (without zeros)
+    Upper truncated geometric distribution (with zeros)
+    Generalized Yule distribution
+    Original Yule distribution
+    
+    
+Likelihood functions
+    Log-likelihood truncated Poisson lognormal distribution
+    Log-likelihood logseries distribution
+    Log-likelihood lower truncated Weibull distribution
+    Log-likelihood of a discrete uniform distribution with bounds [low, high]
+    Log-likelihood of a geomtric distribution
+    Log-likelihood of a negative binomial dstribution (truncated at 1)
+    Log-likelihood of a discrete gamma distribution
+    Log-likelihood of the generalized Yule distribution
+    Log-likelihood of the original Yule-Simon distribution
+    
+
+"""
 
 from __future__ import division
 import sys
@@ -7,6 +33,7 @@ from numpy import exp, histogram, log, matlib, sort, pi, std, mean
 import numpy as np
 from scipy import integrate, stats, optimize, special
 from scipy.stats import rv_discrete, rv_continuous
+from scipy.optimize import bisect
 from scipy.integrate import quad
 
 #._rvs method is not currently available for pln.
@@ -25,7 +52,7 @@ class pln_gen(rv_discrete):
     (http://www.nceas.ucsb.edu/projects/11121)
     
     """    
-    def _pmf(self, x, mu, sigma, lower_trunc, approx_cut = 10, full_output = 0):
+    def _pmf(self, x, mu, sigma, lower_trunc, approx_cut = 10):
         def untrunc_pmf(x_i, mu, sigma):
             pmf_i = 1e-120
             if sigma > 0:
@@ -57,17 +84,10 @@ class pln_gen(rv_discrete):
                         ub = x_i
                     term1 = ((2 * pi * sigma ** 2) ** -0.5)/ factorial(x_i)
                     #integrate low end where peak is so it finds peak
-                    term2a = integrate.quad(lambda x: ((x ** (x_i - 1)) * 
-                                                       (exp(-x)) * 
-                                                       exp(-(log(x) - mu) ** 2 / 
-                                                           (2 * sigma ** 2))), 0,
-                                                   ub, full_output = full_output, limit = 100)
+                    eq = lambda t: np.exp(t * x_i - np.exp(t) - (t - mu) ** 2 / 2 / (sigma ** 2))
+                    term2a = integrate.quad(eq, -float('inf'), np.log(ub), full_output = 0, limit = 500)
                     #integrate higher end for accuracy and in case peak moves
-                    term2b = integrate.quad(lambda x: ((x ** (x_i - 1)) * 
-                                                       (exp(-x)) * exp(-(log(x) - mu) ** 
-                                                                       2/ (2 * sigma ** 
-                                                                           2))), ub,
-                                                   float('inf'), full_output = full_output, limit = 100)
+                    term2b = integrate.quad(eq, np.log(ub), float('inf'), full_output = 0, limit = 500)
                     Pr = term1 * term2a[0]
                     Pr_add = term1 * term2b[0]  
                     if Pr + Pr_add > 0: 
@@ -88,11 +108,11 @@ class pln_gen(rv_discrete):
             pmf.append(pmf_i)
         return np.array(pmf)
     
-    def _cdf(self, x, mu, sigma, lower_trunc, approx_cut = 10, full_output = 0):
+    def _cdf(self, x, mu, sigma, lower_trunc, approx_cut = 10):
         x = np.array(x)
         cdf = []
         for x_i in x:
-            cdf.append(sum(self.pmf(range(int(x_i) + 1), mu, sigma, lower_trunc)))
+            cdf.append(sum(self.pmf(range(int(x_i) + 1), mu[0], sigma[0], lower_trunc[0])))
         return np.array(cdf)
 
     def _argcheck(self, *args):
@@ -114,18 +134,42 @@ class trunc_logser_gen(rv_discrete):
     """
     def _pmf(self, x, p, upper_bound):
         x = np.array(x)
-        if p < 1:
+        #return stats.logser.pmf(x, p) / stats.logser.cdf(upper_bound, p)
+        if p[0] < 1:
             return stats.logser.pmf(x, p) / stats.logser.cdf(upper_bound, p)
         else:
-            x = np.array(x)
-            ivals = np.arange(1, upper_bound + 1)
-            normalization = sum(p ** ivals / ivals)
-            pmf = (p ** x / x) / normalization
+            ivals = np.arange(1, upper_bound[0] + 1)
+            normalization = sum(p[0] ** ivals / ivals)
+            pmf = (p[0] ** x / x) / normalization
             return pmf
 
+    def _cdf(self, x, p, upper_bound):
+        x = np.array(x)
+        if p[0] < 1:
+            return stats.logser.cdf(x, p) / stats.logser.cdf(upper_bound, p)
+        else:
+            cdf_list = [sum(self.pmf(range(1, int(x_i) + 1), p[0], upper_bound[0])) for x_i in x]
+            return np.array(cdf_list)
+    
+    def _rvs(self, p, upper_bound):
+        out = []
+        if p < 1:
+            for i in range(self._size):
+                rand_logser = stats.logser.rvs(p)
+                while rand_logser > upper_bound:
+                    rand_logser = stats.logser.rvs(p)
+                out.append(rand_logser)
+        else:
+            rand_list = stats.uniform.rvs(size = self._size)
+            for rand_num in rand_list:
+                y = lambda x: self.cdf(x, p, upper_bound) - rand_num
+                if y(1) > 0: out.append(1)
+                else: out.append(int(round(bisect(y, 1, upper_bound))))
+        return np.array(out)
+
 trunc_logser = trunc_logser_gen(a=1, name='trunc_logser',
                                 longname='Upper truncated logseries',
-                                shapes="upper_bound",
+                                shapes="p, upper_bound",
                                 extradoc="""Truncated logseries
                                 
                                 Upper truncated logseries distribution
@@ -229,11 +273,11 @@ trunc_weibull = trunc_weibull_gen(name = 'trunc_weibull', longname = 'Lower trun
                                   shapes = 'k, lmd, lower_bound')
 
 class trunc_geom_gen(rv_discrete):
-    """Upper truncated geometric distribution"""
+    """Upper truncated geometric distribution (without zeros)"""
     def _pmf(self, x, p, upper_bound):
         x = np.array(x)
-        pdf = (1 - p) ** (x - 1) * p / (1 - (1 - p) ** upper_bound)
-        return pdf
+        pmf = (1 - p) ** (x - 1) * p / (1 - (1 - p) ** upper_bound)
+        return pmf
     
     def _cdf(self, x, p, upper_bound):
         x = np.array(x)
@@ -249,7 +293,7 @@ class trunc_geom_gen(rv_discrete):
         rand_num = stats.geom.rvs(p, size = self._size)
         rand_num = rand_num[rand_num <= upper_bound]
         while (len(rand_num) < self._size):
-            rand_new = stats.frechet_r.rvs(p)
+            rand_new = stats.geom.rvs(p)
             if rand_new <= upper_bound:
                 rand_num = np.append(rand_num, rand_new)
         return rand_num
@@ -261,7 +305,33 @@ class trunc_geom_gen(rv_discrete):
 trunc_geom = trunc_geom_gen(name = 'trunc_geom', longname = 'Upper truncated geometric', 
                                   shapes = 'p, upper_bound')
 
-def pln_ll(x, mu, sigma, lower_trunc = True, full_output = 0):
+class trunc_geom_with_zeros_gen(rv_discrete):
+    """Upper truncated geometric distribution (with zeros)"""
+    def _pmf(self, x, p, upper_bound):
+        x = np.array(x)
+        pmf = (1 - p) ** x * p / (1 - (1 - p) ** (upper_bound + 1))
+        return pmf
+    
+    def _cdf(self, x, p, upper_bound):
+        x = np.array(x)
+        cdf = (1 - (1 - p) ** (x + 1)) / (1 - (1 - p) ** (upper_bound + 1))
+        return cdf
+    
+    def _ppf(self, cdf, p, upper_bound):
+        cdf = np.array(cdf)
+        x = np.log(1 - cdf * (1 - (1 - p) ** (upper_bound + 1))) /\
+            np.log(1 - p) - 1
+        return np.ceil(x)
+    
+    def _argcheck(self, p, upper_bound):
+        cond = (p > 0) & (upper_bound >= 1) 
+        return cond
+
+trunc_geom_with_zeros = trunc_geom_with_zeros_gen(name = 'trunc_geom_with_zeros', 
+                                                  longname = 'Upper truncated geometric with zeros', 
+                                                  shapes = 'p, upper_bound')
+
+def pln_ll(x, mu, sigma, lower_trunc = True):
     """Log-likelihood of a truncated Poisson lognormal distribution
     
     Method derived from Bulmer 1974 Biometrics 30:101-110    
@@ -282,7 +352,7 @@ def pln_ll(x, mu, sigma, lower_trunc = True, full_output = 0):
     cts = histogram(x, bins = range(1, max(x) + 2))
     observed_abund_vals = cts[1][cts[0] != 0]
     counts = cts[0][cts[0] != 0]
-    plik = pln.logpmf(observed_abund_vals, mu, sigma, lower_trunc, full_output = full_output)
+    plik = pln.logpmf(observed_abund_vals, mu, sigma, lower_trunc)
     lik_list = np.array([], dtype = float)
     for i, count in enumerate(counts):
         lik_list = np.append(lik_list, count * plik[i])
@@ -304,7 +374,7 @@ def logser_ll(x, p, upper_trunc = False, upper_bound = None):
         return sum(stats.logser.logpmf(x, p))
 
 def trunc_weibull_ll(x, k, lmd, lower_bound):
-    """Log-likelihood of the Weibull distributed lower truncated at lower_bound"""
+    """Log-likelihood of the Weibull distribution lower truncated at lower_bound"""
     return sum(trunc_weibull.logpdf(x, k, lmd, lower_bound))
 
 def disunif_ll(ab, low, high):
@@ -313,12 +383,12 @@ def disunif_ll(ab, low, high):
     return n * log(1 / (high - low + 1))
 
 def geom_ll(ab, p):
-    """Log-likelihood of a geomtric distribution"""
+    """Log-likelihood of a geometric distribution"""
     return sum(stats.geom.logpmf(ab, p))
 
 def negbin_ll(ab, n, p):
     """Log-likelihood of a negative binomial dstribution (truncated at 1)"""
-    return sum(log(stats.nbinom.pmf(ab, n, p) / (1 - stats.nbinom.pmf(0, n, p))))
+    return sum(stats.nbinom.logpmf(ab, n, p)) - len(ab) * log(1 - stats.nbinom.pmf(0, n, p))
 
 def dis_gamma_ll(ab, k, theta):
     """Log-likelihood of a discrete gamma distribution
@@ -333,18 +403,17 @@ def dis_gamma_ll(ab, k, theta):
     C = 1 / gamma_sum
     return sum(log(stats.gamma.pdf(ab, k, scale = theta) * C))
 
-def gen_yule_ll(ab, a, b):
+def gen_yule_ll(ab, a, rho):
     """Log-likelihood of the Yule-Simon distribution.
     
     Follow the configuration of generalized Yule distribution in Nee 2003. 
-    The configuration on wikipedia and in Simon 1955 is the species case 
-    where a = 1 and b = rho. 
+    The configuration on wikipedia and in Simon 1955 is the special case 
+    where a = 1. 
     
     """
-    ll = 0
+    ll = len(ab) * (log(rho) + special.gammaln(a + rho) - special.gammaln(a))
     for ab_i in ab: 
-        ll += log(b * special.gamma(a + b) * special.gamma(a + ab_i - 1) / 
-                  special.gamma(a) /  special.gamma(a + b + ab_i))
+        ll += special.gammaln(a + ab_i -1) - special.gammaln(a + rho + ab_i)
     return ll
 
 def yule_ll(ab, rho):
@@ -364,7 +433,7 @@ def pln_solver(ab, lower_trunc = True):
     mu0 = mean(log(ab))
     sig0 = std(log(ab))
     def pln_func(x): 
-        return -pln_ll(ab, x[0], x[1], lower_trunc, full_output = 1)
+        return -pln_ll(ab, x[0], x[1], lower_trunc)
     mu, sigma = optimize.fmin(pln_func, x0 = [mu0, sig0], disp = 0)
     return mu, sigma
 
@@ -380,6 +449,17 @@ def trunc_logser_solver(ab):
                         min((sys.float_info[0] / S) ** (1 / N), 2), xtol = 1.490116e-08)
     return p
 
+def trunc_geom_solver(ab, upper_bound):
+    """Given abundance data, solve for MLE of upper-truncated geometric distribution parameter p"""
+    BOUNDS = [0, 1]
+    DIST_FROM_BOUND = 10 ** -10
+    S = len(ab)
+    N = sum(ab)
+    y = lambda x: (N * (S-1) * (1-x) + S) * x ** upper_bound + N * (1-x) - S
+    one_minus_p = optimize.bisect(y, 1 - S/N - DIST_FROM_BOUND, BOUNDS[1] - DIST_FROM_BOUND, 
+                                  xtol = 1.490116e-16)
+    return 1 - one_minus_p
+
 def trunc_expon_solver(x, lower_bound):
     """Given a random sample and lower bound, 
     
@@ -420,19 +500,63 @@ def dis_gamma_solver(ab):
     return k, theta 
 
 def gen_yule_solver(ab):
-    """Given abundance data, solve for MLE of generalized Yule distribution parameters a and b"""
-    p1 = ab.count(1) / len(ab)
-    a0 = 1                    
-    b0 = p1 / (1-p1) * a0  # Initial guess based on expected frequency of singletons. NOT STABLE. 
-    def gen_yule_func(x):
-        return -gen_yule_ll(ab, x[0], x[1])
-    a, b = optimize.fmin(gen_yule_func, x0 = [a0, b0])
-    return a, b
-
-def yule_solver(ab):
-    """Given abundance data, solve for MLE of original Yule distribution parameter rho"""
+    """Given abundance data, solve for MLE of generalized Yule distribution parameters a and b(rho)
+    
+    Algorithm extended from Garcia 2011.
+    
+    
+    """
+    a0 = 1
     rho0 = np.mean(ab) / (np.mean(ab) - 1)
-    def yule_func(x):
-        return -yule_ll(ab, x)
-    rho = optimize.fmin(yule_func, x0 = rho0)
-    return rho
\ No newline at end of file
+    tol = 1.490116e-08
+    loop_end = False
+    count_one = len([k for k in ab if k == 1])
+    ab_not_one = [k for k in ab if k != 1]
+    max_iter = 1000 # maximum number of iterations 
+    i = 0
+    while (not loop_end) and (i < max_iter):
+        rho1 = len(ab) / sum([sum([1 / (rho0 + j + a0) for j in range(0, k)]) for k in ab])
+        func_a = lambda a: 1 / (a + rho1) * count_one + sum([1 / (a+rho1+k-1) - \
+                                                             sum([rho1/(a+rho1+m)/(a+m) for m in range(0, k - 1)])\
+                                                             for k in ab_not_one])
+        try:
+            a1 = optimize.newton(func_a, a0, maxiter = 500)
+        except RuntimeError:
+            i = max_iter
+            break
+        loop_end = (abs(rho1 - rho0) < tol) * (abs(a1 - a0) < tol)
+        a0, rho0 = a1, rho1
+        i += 1
+    if i < max_iter: return a1, rho1
+    else: 
+        print "Failed to converge."
+        return None, None
+    
+def yule_solver(ab):
+    """Given abundance data, solve for MLE of original Yule distribution parameter rho
+    
+    Algorithm from Garcia 2011.
+    
+    """
+    rho0 = np.mean(ab) / (np.mean(ab) - 1)
+    tol = 1.490116e-08
+    loop_end = False
+    while not loop_end:
+        rho1 = len(ab) / sum([sum([1 / (rho0 + j) for j in range(1, k+1)]) for k in ab])
+        loop_end = (abs(rho1 - rho0) < tol)
+        rho0 = rho1
+    return rho1
+
+def xsquare_pdf(x, dist, *pars):
+    """Calculates the pdf for x, given the distribution of variable Y = sqrt(X) 
+    
+    and a given value x. 
+    
+    """
+    x = np.array(x)
+    return 1 / x ** 0.5 * dist.pdf(x ** 0.5, *pars) / 2 
+
+def ysquareroot_pdf(y, dist, *pars):
+    """Calculates the pdf for y, given the distribution of variable X = Y^2 and a given value y."""
+    y = np.array(y)
+    return 2 * dist.pdf(y ** 2, *pars) * y
